from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_query.extension import List
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user_joined_voice_channel import UserJoinedVoiceChannel

from bot_data.abc.user_joined_voice_channel_abc import UserJoinedVoiceChannelRepositoryABC


class UserJoinedVoiceChannelRepositoryService(UserJoinedVoiceChannelRepositoryABC):

    def __init__(self, logger: LoggerABC, db_context: DatabaseContextABC, users: UserRepositoryABC):
        self._logger = logger
        self._context = db_context

        self._users = users

        UserJoinedVoiceChannelRepositoryABC.__init__(self)

    def get_user_joined_voice_channels(self) -> List[UserJoinedVoiceChannel]:
        joins = List(UserJoinedVoiceChannel)
        self._logger.trace(__name__, f'Send SQL command: {UserJoinedVoiceChannel.get_select_all_string()}')
        results = self._context.select(UserJoinedVoiceChannel.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f'Get user-joined-voice-channel with id {result[0]}')
            joins.append(UserJoinedVoiceChannel(
                self._users.get_user_by_id(result[1]),
                result[2],
                result[3],
                result[4],
                result[5],
                id=result[0]
            ))

        return joins
    
    def get_user_joined_voice_channel_by_id(self, id: int) -> UserJoinedVoiceChannel:
        self._logger.trace(__name__, f'Send SQL command: {UserJoinedVoiceChannel.get_select_by_id_string(id)}')
        result = self._context.select(UserJoinedVoiceChannel.get_select_by_id_string(id))[0]
        return UserJoinedVoiceChannel(
            self._users.get_user_by_id(result[1]),
            result[2],
            result[3],
            result[4],
            result[5],
            id=result[0]
        )
    
    def get_user_joined_voice_channels_by_user_id(self, user_id: int) -> List[UserJoinedVoiceChannel]:
        joins = List(UserJoinedVoiceChannel)
        self._logger.trace(__name__, f'Send SQL command: {UserJoinedVoiceChannel.get_select_by_user_id_string(user_id)}')
        results = self._context.select(UserJoinedVoiceChannel.get_select_by_user_id_string(user_id))
        for result in results:
            joins.append(UserJoinedVoiceChannel(
                self._users.get_user_by_id(result[1]),
                result[2],
                result[3],
                result[4],
                result[5],
                id=result[0]
            ))
    
        return joins
    
    def get_active_user_joined_voice_channel_by_user_id(self, user_id: int) -> UserJoinedVoiceChannel:
        self._logger.trace(__name__, f'Send SQL command: {UserJoinedVoiceChannel.get_select_by_user_id_string(user_id)}')
        result = self._context.select(UserJoinedVoiceChannel.get_select_active_by_user_id_string(user_id))[0]
        return UserJoinedVoiceChannel(
            self._users.get_user_by_id(result[1]),
            result[2],
            result[3],
            result[4],
            result[5],
            id=result[0]
        )
    
    def find_active_user_joined_voice_channel_by_user_id(self, user_id: int) -> Optional[UserJoinedVoiceChannel]:
        self._logger.trace(__name__, f'Send SQL command: {UserJoinedVoiceChannel.get_select_by_user_id_string(user_id)}')
        result = self._context.select(UserJoinedVoiceChannel.get_select_active_by_user_id_string(user_id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return UserJoinedVoiceChannel(
            self._users.get_user_by_id(result[1]),
            result[2],
            result[3],
            result[4],
            result[5],
            id=result[0]
        )

    def find_active_user_joined_voice_channels_by_user_id(self, user_id: int) -> List[Optional[UserJoinedVoiceChannel]]:
        self._logger.trace(__name__, f'Send SQL command: {UserJoinedVoiceChannel.get_select_by_user_id_string(user_id)}')
        result = List(UserJoinedVoiceChannel)
        db_results = self._context.select(UserJoinedVoiceChannel.get_select_active_by_user_id_string(user_id))

        for db_result in db_results:
            result.append(UserJoinedVoiceChannel(
                self._users.get_user_by_id(db_result[1]),
                db_result[2],
                db_result[3],
                db_result[4],
                db_result[5],
                id=db_result[0]
            ))

        return result
    
    def add_user_joined_voice_channel(self, user_joined_voice_channel: UserJoinedVoiceChannel):
        self._logger.trace(__name__, f'Send SQL command: {user_joined_voice_channel.insert_string}')
        self._context.cursor.execute(user_joined_voice_channel.insert_string)
    
    def update_user_joined_voice_channel(self, user_joined_voice_channel: UserJoinedVoiceChannel):
        self._logger.trace(__name__, f'Send SQL command: {user_joined_voice_channel.udpate_string}')
        self._context.cursor.execute(user_joined_voice_channel.udpate_string)
    
    def delete_user_joined_voice_channel(self, user_joined_voice_channel: UserJoinedVoiceChannel):
        self._logger.trace(__name__, f'Send SQL command: {user_joined_voice_channel.delete_string}')
        self._context.cursor.execute(user_joined_voice_channel.delete_string)
